package ch.ipi.hello.web.form;

import java.util.Locale;
import java.util.UUID;

import javax.annotation.CheckForNull;
import javax.annotation.Nonnull;
import javax.annotation.Nullable;

import com.google.common.base.Function;
import com.google.common.base.Optional;

public class FormularIdentifiers {

  /**
   * UUID to string.
   */
  public static final Function<UUID, String> TO_UPPER_STRING = uuid-> uuid.toString().toUpperCase(Locale.getDefault());

  /**
   * Parses {@link UUID}s.
   */
  public static final Function<CharSequence, UUID> PARSE = input-> {
    final Optional<UUID> id = parseId(input);
    return id.get();
  };

  private static final int UUID_MAX_LENGTH = 36;

  private FormularIdentifiers() {
    super();
  }

  /**
   * Creates an id.
   * @return the id
   */
  @Nonnull
  public static UUID create() {
    return UUID.randomUUID();
  }

  /**
   * Parse the id.
   * @param id the id
   * @return the parsed id
   */
  @Nonnull
  public static Optional<UUID> parseId(@Nullable final CharSequence id) {
    if (id == null || id.length() > UUID_MAX_LENGTH) {
      return Optional.absent();
    }
    try {
      return Optional.of(UUID.fromString(id.toString()));
    } catch (@SuppressWarnings("unused") final IllegalArgumentException e) {
      return Optional.absent();
    }
  }

  /**
   * Formats the id as string.
   * @param id the id
   * @return the string or {@code null}
   */
  @CheckForNull
  public static String toString(@Nullable final UUID id) {
    return id == null ? null : FormularIdentifiers.TO_UPPER_STRING.apply(id);
  }

}
