package ch.ipi.hello.web.form.instance;

import java.util.List;
import java.util.UUID;

import javax.annotation.Nonnull;
import javax.ws.rs.DELETE;
import javax.ws.rs.GET;
import javax.ws.rs.Path;
import javax.ws.rs.Produces;
import javax.ws.rs.core.Response;

import ch.ipi.hello.web.AbstractFormApplicationResource;
import ch.ipi.hello.web.form.FormularData;
import ch.ipi.hello.web.form.GenericFormResource;

public abstract class AbstractFormInstanceResource<C> extends AbstractFormApplicationResource {

  private C scope;

  public GenericFormResource getForm() {
    return factory(GenericFormResource.class).get();
  }

  /**
   * Gets the scope.
   * @return the scope
   */
  protected C getScope() {
    return this.scope;
  }

  /**
   * Sets the scope.
   * @param scope The scope to set
   */
  protected void setScope(final C scope) {
    this.scope = scope;
  }

  /**
   * Gets the ids.
   * @return the ids
   */
  @Nonnull
  public abstract List<UUID> getIDs();

  /**
   * Gets an appropriate {@link FormInstanceResource} for the form type.
   * @param instance the form
   * @return the resource dispatcher
   */
  @Nonnull
  protected FormInstanceResource single(final FormularData instance) {
    return this.getForm().single(instance);
  }

  /**
   * Gets an appropriate {@link FormInstanceResource} for the form type.
   * @param instance the form
   * @return the resource dispatcher
   */
  @Nonnull
  protected FormInstanceResource single(final UUID instance) {
    return this.getForm().single(instance);
  }

  /**
   * Shows the type.
   * @return the type
   */
  @GET
  @Path("type")
  public Response showType() {
    return representations("form type").build();
  }

  /**
   * Exports to a ZIP.
   * @return the zip
   */
  @GET
  @Path("export")
  @Produces("application/zip")
  public Response export() {
    return null;
  }

  /**
   * Deletes the instances.
   * @return the delete count
   */
  @DELETE
  public abstract Response delete();

}
