package ch.ipi.hello.web.form.instance.collection;

import static javax.ws.rs.core.Response.Status.NOT_FOUND;

import java.util.List;
import java.util.UUID;

import javax.ws.rs.GET;
import javax.ws.rs.PUT;
import javax.ws.rs.Path;
import javax.ws.rs.PathParam;
import javax.ws.rs.core.Response;

import ch.ipi.hello.web.form.instance.AbstractFormInstanceResource;
import ch.ipi.hello.web.form.instance.FormInstanceResource;

import com.google.common.collect.ImmutableList;

public class GenericFormInstanceCollectionResource extends AbstractFormInstanceResource<List<UUID>>
    implements FormInstanceCollectionResource {

  /**
   * {@inheritDoc}
   */
  @Override
  public GenericFormInstanceCollectionResource getResource() {
    return this;
  }

  @Override
  public List<UUID> getIDs() {
    return getScope();
  }

  /**
   * Sets the ids.
   * @param ids the ids
   */
  @Override
  public void setIDs(final Iterable<UUID> ids) {
    setScope(ImmutableList.copyOf(ids));
  }

  /**
   * The ids.
   * @return the ids
   */
  @GET
  public Response listIDs() {
    return representations(getIDs()).build();
  }

  /**
   * Locates an instance by index.
   * @param idx the index
   * @return the instance
   */
  @Path("{idx:-?[0-9]+}")
  @SuppressWarnings("PMD.PreserveStackTrace")
  public FormInstanceResource locateInstanceAtIndex(@PathParam("idx") final int idx) {
    final List<UUID> ids = getIDs();
    final int n = ids.size();
    if (n == 0) {
      throw throwMessage(NOT_FOUND, "empty list, no indexing possible");
    }
    try {
      return single(ids.get(idx < 0 ? n + idx : idx));

    } catch (final IndexOutOfBoundsException ignored) {
      throw throwMessage(NOT_FOUND, "valid indicies from %s to %s", -n, n - 1);
    }
  }

  /**
   * Transitions all form instances of this collection to the requested state.
   * @param requestedState the requested state
   */
  @PUT
  @Path("state")
  public void transitionAllToState(final String requestedState) {

  }

  /**
   * {@inheritDoc}
   */
  @Override
  public Response delete() {
    final int totalCount = 42;

    return representations(totalCount).build();
  }

}
